//Florian Krmer 
//Projekt Stoppuhr

org 0000h

jmp Main				//Freimachen der Interrupt-Einsprungadress

org 000Bh 				//Einsprungadresse Timer0-Interrupt

ljmp ISR_Timer0

org 001Bh				//Einsprungadresse Timer1-Interrupt

ljmp ISR_Timer1


org 0100h

Main:
			

call init		  		//Initialisierung der verwendeten Register
call timer_cfg			//Timer konfigurieren
call ram_belegen		//Belegen des Rams mit den Zhlvariablen
loop:

call Ausgabe

call time				//Unterprogramm zur Zeitberechnung aufrufen

jmp loop				//Endlosschleife


ISR_Timer0:				//Interrupt-Service-Routine Timer0

mov TL0, #0B0h			//Timer0 vorladen
mov TH0, #3Ch			//65536-50000=15536 --> 3CB0h (50ms)
mov R0, #1				//Freigabe zur Zeitberechnung
reti


ISR_Timer1:				//Interrupt-Service-Routine Timer1

mov TL1, #078h			//Timer1 vorladen
mov TH1, #0ECh			//65536-5000=60536 --> EC78h (5ms)
mov R2, #1

reti

init:					//Initialisierung der verwendeten Register

mov R0, #0				//Zhler von 0-20 --> 20Hz Interruptfrequenz (50ms) *20 =1s
mov R1, #0				//"Zhlfreigabe" durch den Interrupt
mov R2, #0				//Ausgabefreigabe durch den Interrupt
mov sec_one, #0			//Segment rechts (Sekunden Einer)
mov sec_ten, #0			//Segment Mitte recht (Sekunden Zehner)
mov min_one, #0			//Segment Mitte links (Minuten Einer)
mov min_ten, #0			//Segment links (Minuten Zehner)
mov A, #0
mov P2, #0
mov P3, #0


ret

timer_cfg:				//Timer konfigurieren

setb EA					//Interrupts generell freigeben			
setb ET0				//Timer0-Interrupt freigeben
setb ET1				//Timer1-Interrupt freigeben
mov TMOD, #00010001b	//Betriebsmode festlegen (jeweils Betriebsmode1)
setb TR0				//Timer0 luft
setb TR1				//Timer1 luft

mov TL0, #0B0h			//Timer0 vorladen
mov TH0, #3Ch			//65536-50000=15536 --> 3CB0h (50ms)

mov TL1, #078h			//Timer1 vorladen
mov TH1, #0ECh			//65536-5000=60536 --> EC78h (5ms)

ret

ram_belegen:


dseg at 0030h

 
sec_one:	ds 1
sec_ten:	ds 1
min_one:	ds 1
min_ten:	ds 1

cseg

ret


time:

cjne R0,#1d,time			//Die Zeit kann nur berechnet werden,
mov R0, #0d					//wenn der Timer0-Interrupt ausgelst wurde

inc R1						//Zhler von 0-20 --> 20Hz Interruptfrequenz (50ms) *20 =1s
cjne R1,#20d,weiter			//fals 20 nicht erreicht wurde, Unterprogramm beenden
mov R1,#0d					//nach berlauf Register wieder 0 setzen

inc sec_one
mov R3, sec_one				//Segment rechts (Sekunden Einer)
cjne R3,#10d,weiter			//Zhlt jede Sekunde 1 hoch. Unter 10 Sprung 
							//aus dem Unterprogramm 
mov sec_one,#0d				//Fals 10 Sekunden erreicht wurden, wieder auf 0 Rcksetzen
							//und weiter mit Sekunden Zehner fortfahren.

inc sec_ten	
mov R3, sec_ten				//Segment Mitte rechts (Sekunden Zehner)
cjne R3,#6d, weiter			//Zhlt jede 10 Sekunden 1 hoch. Unter 6 Sprung 
							//aus dem Unterprogramm 
mov sec_ten,#0d				//Fals 60 Sekunden erreicht wurden, wieder auf 0 Rcksetzen
							//und weiter mit Minuten Einer fortfahren.

inc min_one
mov R3, min_one				//Segment Mitte links (Minuten Ener)
cjne R3,#10d, weiter		//Zhlt jede 60 Sekunden 1 hoch. Unter 10 Sprung 
							//aus dem Unterprogramm
mov min_one,#0d				//Fals 10 Minuten erreicht wurden, wieder auf 0 Rcksetzen 
							//und weiter mit Minuten Zehner fortfahren.

inc min_ten
mov R3, min_ten				//Segment links (Minuten Zehner)
cjne R3,#6d, weiter			//Zhlt jede 10 Minuten 1 hoch. Unter 60 Sprung 
							//aus dem Unterprogramm
mov min_ten,#0d				//Fals 10 Minuten erreicht wurden, wieder auf 0 Rcksetzen
jmp weiter					//und Fehlermeldung ausgeben, da bergeloffen.

weiter:
ret							//Sprung aus dem Unterprogramm

Ausgabe:					//Ausgabe der Zeit an die 7.Segment-Anzeige

cjne R2, #1d, Ausgabe		//Es kann nur ausgegeben werden,
mov R2, #0d					//wenn der Timer1-Interrupt ausgelst wurde

mov A, sec_one				//Sekunden Einer in den Akku schreiben
call Seg_Codierung			//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A					//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00001110b			//Freigabe der rechten 7.Segment-Anzeige
call zeit					//Zeitschleife aufrufen


mov A, sec_ten		  		//Sekunden Zehner in den Akku schreiben
call Seg_Codierung			//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A					//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00001101b			//Freigabe der mittleren rechten 7.Segment-Anzeige
call zeit					//Zeitschleife aufrufen


mov A, min_one			   	//Minuten Einer in den Akku schreiben
call Seg_Codierung	   		//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A			   		//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00001011b	   		//Freigabe der mittleren linken 7.Segment-Anzeige
call zeit					//Zeitschleife aufrufen


mov A, min_ten		   		//Minuten Zehner in den Akku schreiben
call Seg_Codierung	   		//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A			   		//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00000111b			//Freigabe der linken 7.Segment-Anzeige
call zeit			  		//Zeitschleife aufrufen


ret


Seg_Codierung:				//wandelt die berechnete Zeit (Zahl) in eine 7-Segment-Zahl	

	mov dptr, #Tab
	movc A, @A+dptr
	ret

Tab:
db 00111111b, 00000110b, 01011011b, 01001111b, 01100110b 
db 01101101b, 01111101b, 00000111b, 01111111b, 01101111b



Zeit:						//Zeitschleife

		mov A, #0d
Zeit1:	dec A
		cjne A, #0d, Zeit1
		mov A, #0d
Zeit2:	dec A
		cjne A, #0d, Zeit2
		mov A, #0d
		mov A, #0d
Zeit3:	dec A
		cjne A, #0d, Zeit3
		mov A, #0d
		ret 

end
